using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace Schraffur
{
   public partial class Form1 : Form
   {
      Stack<Hatching> undoStack = new Stack<Hatching>();
      Hatching hatchingConfig = new Hatching();
      Bitmap currentImage = new Bitmap(1, 1);
      const int stdSize = 256;

      bool isLoading = true;

      void configToControls()
      {
         isLoading = true;
         nudBackColorAlpha.Value = hatchingConfig.BackColorAlpha;
         nudLineColorAlpha.Value = hatchingConfig.LineColorAlpha;
         nudThickness.Value = hatchingConfig.LineThickness;
         nudSpacing.Value = hatchingConfig.LineSpacing;
         isLoading = false;
      }

      public Form1()
      {
         InitializeComponent();

         configToControls();

         picMap.Visible = false;
         picSymbols.Visible = false;
         modified();
      }

      private void modified()
      {
         undoStack.Push(hatchingConfig.Clone());
         currentImage = paint();
         picHatching.Image = currentImage;
         cmdUndo.Enabled = true;
      }

      #region [ Hintergrudfarbe ]
      private void cmdColorBackground_Click(object sender, EventArgs e)
      {
         colorDialog.Color = hatchingConfig.BackColor;
         colorDialog.Title = "Hintergrundfarbe";
         if (colorDialog.ShowDialog() == DialogResult.OK)
         {
            hatchingConfig.BackColor = colorDialog.Color;
            modified();
         }
      }
      private void nudBackColorAlpha_ValueChanged(object sender, EventArgs e)
      {
         if (isLoading)
            return;

         hatchingConfig.BackColorAlpha = (int)nudBackColorAlpha.Value;
         modified();
      }

      #endregion

      #region [ Linienfarbe ]
      private void cmdLineColor_Click(object sender, EventArgs e)
      {
         colorDialog.Color = hatchingConfig.LineColor;
         colorDialog.Title = "Linienfarbe";

         if (colorDialog.ShowDialog() == DialogResult.OK)
         {
            hatchingConfig.LineColor = colorDialog.Color;
            modified();
         }
      }
      private void nudLineColorAlpha_ValueChanged(object sender, EventArgs e)
      {
         if (isLoading)
            return;

         hatchingConfig.LineColorAlpha = (int)nudLineColorAlpha.Value;
         modified();
      }
      #endregion

      #region [Linien-Typ ]
      private void nudThickness_ValueChanged(object sender, EventArgs e)
      {
         if (isLoading)
            return;

         hatchingConfig.LineThickness = (int)nudThickness.Value;
         modified();
      }

      private void nudSpacing_ValueChanged(object sender, EventArgs e)
      {
         if (isLoading)
            return;

         hatchingConfig.LineSpacing = (int)nudSpacing.Value;
         modified();
      }
      #endregion

      private void cmdUndo_Click(object sender, EventArgs e)
      {
         undoStack.Pop();
         hatchingConfig = undoStack.Peek();

         if (undoStack.Count == 1)
            cmdUndo.Enabled = false;
         configToControls();
         currentImage = paint();
         picHatching.Image = currentImage;
      }

      #region [ Zeichnen ]
      private Bitmap paint()
      {
         int gap = hatchingConfig.LineSpacing + hatchingConfig.LineThickness; // Breite einer Einheit (Strich + Abstand)

         Pen pen = new Pen(Color.FromArgb(hatchingConfig.LineColorAlpha * 255 / 100, hatchingConfig.LineColor), hatchingConfig.LineThickness);
         SolidBrush brush = new SolidBrush(Color.FromArgb(hatchingConfig.BackColorAlpha * 255 / 100, hatchingConfig.BackColor));

         // Abmessung fr Kachel berechnen
         int cnt = (int)Math.Floor((double)stdSize / gap); // Anzahl Einheiten, die Vollstndig in stdSize passen
         int size = cnt * gap;
         picHatching.Width = size;
         picHatching.Height = size;
         Bitmap bmp = new Bitmap(picHatching.Width, picHatching.Height);
         Graphics g = Graphics.FromImage(bmp);

         g.FillRectangle(brush, 0, 0, picHatching.Height, picHatching.Width);

         if (hatchingConfig.Type == HatchingType.Lines && hatchingConfig.Orientation == Orientation.Vertical
            || hatchingConfig.Type == HatchingType.Crosses && hatchingConfig.Orientation == Orientation.Vertical
            || hatchingConfig.Type == HatchingType.Crosses && hatchingConfig.Orientation == Orientation.Horizontal)
            drawVertical(g, pen, cnt, gap);

         if (hatchingConfig.Type == HatchingType.Lines && hatchingConfig.Orientation == Orientation.Horizontal
            || hatchingConfig.Type == HatchingType.Crosses && hatchingConfig.Orientation == Orientation.Vertical
            || hatchingConfig.Type == HatchingType.Crosses && hatchingConfig.Orientation == Orientation.Horizontal)
            drawHorizontal(g, pen, cnt, gap);

         if (hatchingConfig.Type == HatchingType.Lines && hatchingConfig.Orientation == Orientation.Diagonal
            || hatchingConfig.Type == HatchingType.Crosses && hatchingConfig.Orientation == Orientation.Diagonal
            || hatchingConfig.Type == HatchingType.Crosses && hatchingConfig.Orientation == Orientation.AntiDiagonal)
            drawDiagonal(g, pen, cnt, gap);

         if (hatchingConfig.Type == HatchingType.Lines && hatchingConfig.Orientation == Orientation.AntiDiagonal
            || hatchingConfig.Type == HatchingType.Crosses && hatchingConfig.Orientation == Orientation.Diagonal
            || hatchingConfig.Type == HatchingType.Crosses && hatchingConfig.Orientation == Orientation.AntiDiagonal)
            drawAntidiagonal(g, pen, cnt, gap);

         return bmp;
      }

      void drawVertical(Graphics g, Pen pen, int cnt, int gap)
      {
         int x = (int)(hatchingConfig.LineThickness / 2); // Position des 1. Striches. Graphics zeichnet die Mitte(!) des Striches an diese Position.

         for (int i = 0; i < cnt; i++)
         {
            g.DrawLine(pen, i * gap + x, 0, i * gap + x, picHatching.Height);
         }
      }

      void drawHorizontal(Graphics g, Pen pen, int cnt, int gap)
      {
         int x = (int)(hatchingConfig.LineThickness / 2); // Position des 1. Striches. Graphics zeichnet die Mitte(!) des Striches an diese Position.

         for (int i = 0; i < cnt; i++)
         {
            g.DrawLine(pen, 0, i * gap + x, picHatching.Width, i * gap + x);
         }
      }

      void drawDiagonal(Graphics g, Pen pen, int cnt, int gap)
      {
         int x = (int)(hatchingConfig.LineThickness / 2); // Position des 1. Striches. Graphics zeichnet die Mitte(!) des Striches an diese Position.

         for (int i = -cnt; i < cnt; i++)
         {
            g.DrawLine(pen, i * gap + x, 0, i * gap + x + picHatching.Width, picHatching.Height);
         }
      }

      void drawAntidiagonal(Graphics g, Pen pen, int cnt, int gap)
      {
         int x = (int)(hatchingConfig.LineThickness / 2); // Position des 1. Striches. Graphics zeichnet die Mitte(!) des Striches an diese Position.

         for (int i = 0; i < 2 * cnt; i++)
         {
            g.DrawLine(pen, i * gap + x, 0, i * gap + x - picHatching.Width, picHatching.Height);
         }
      }
      #endregion

      #region [ Hintergrund ]
      private void rbBackgroundBlank_CheckedChanged(object sender, EventArgs e)
      {
         picHatching.BackgroundImage = null;
      }

      private void rbBackgroundMap_CheckedChanged(object sender, EventArgs e)
      {
         picHatching.BackgroundImage = picMap.BackgroundImage;
      }

      private void rbBackgroundSymbols_CheckedChanged(object sender, EventArgs e)
      {
         picHatching.BackgroundImage = picSymbols.BackgroundImage;
      }
      #endregion

      #region [ Orientierung ]     
      private void rbVertical_CheckedChanged(object sender, EventArgs e)
      {
         if (isLoading)
            return;

         hatchingConfig.Orientation = Orientation.Vertical;
         modified();
      }

      private void rbHorizontal_CheckedChanged(object sender, EventArgs e)
      {
         if (isLoading)
            return;

         hatchingConfig.Orientation = Orientation.Horizontal;
         modified();
      }

      private void rbDiagonal_CheckedChanged(object sender, EventArgs e)
      {
         if (isLoading)
            return;

         hatchingConfig.Orientation = Orientation.Diagonal;
         modified();
      }

      private void rbAntiDiagonal_CheckedChanged(object sender, EventArgs e)
      {
         if (isLoading)
            return;

         hatchingConfig.Orientation = Orientation.AntiDiagonal;
         modified();
      }
      #endregion

      #region [ Schraffur-Typ ]
      private void rbLines_CheckedChanged(object sender, EventArgs e)
      {
         if (isLoading)
            return;

         hatchingConfig.Type = HatchingType.Lines;
         modified();
      }

      private void rbCrosses_CheckedChanged(object sender, EventArgs e)
      {
         if (isLoading)
            return;

         hatchingConfig.Type = HatchingType.Crosses;
         modified();
      }
      #endregion

      #region [ Kopieren & Speichern ]
      private void cmdClipBoard_Click(object sender, EventArgs e)
      {
         ClipBoardHelper.SetClipboardImage(currentImage, null, null);
      }

      private void cmdSave_Click(object sender, EventArgs e)
      {
         if (saveFileDialog.ShowDialog() == DialogResult.OK)
         {
            var extension = Path.GetExtension(saveFileDialog.FileName);


            switch (extension.ToLower())
            {
               case ".jpg":
                  currentImage.Save(saveFileDialog.FileName, System.Drawing.Imaging.ImageFormat.Jpeg);
                  break;
               case ".png":
                  currentImage.Save(saveFileDialog.FileName, System.Drawing.Imaging.ImageFormat.Png);
                  break;
               default:
                  if (saveFileDialog.FilterIndex == 1)
                     currentImage.Save(saveFileDialog.FileName + ".png", System.Drawing.Imaging.ImageFormat.Png);
                  else
                     currentImage.Save(saveFileDialog.FileName + ".jpg", System.Drawing.Imaging.ImageFormat.Jpeg);
                  break;
            }
         }
      }
      #endregion
   }
}